@@ -4,6 +4,7 @@
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
  * Copyright (C) 2016-2022 Antenore Gatta, Giovanni Panozzo
  * Copyright (C) 2022-2023 Antenore Gatta, Giovanni Panozzo, Hiroyuki Tanaka
+ * Copyright (C) 2023-2024 Hiroyuki Tanaka, Sunil Bhat
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,6 +58,7 @@
 #include "remmina_pref_dialog.h"
 #include "remmina_widget_pool.h"
 #include "remmina_plugin_manager.h"
+#include "remmina_bug_report.h"
 #include "remmina_log.h"
 #include "remmina_icon.h"
 #include "remmina_main.h"
@@ -80,6 +82,7 @@ enum {
 	FILENAME_COLUMN,
 	LABELS_COLUMN,
 	NOTES_COLUMN,
+	STATUS_COLUMN,
 	N_COLUMNS
 };
 
@@ -99,6 +102,7 @@ static GActionEntry app_actions[] = {
 	{ "mpchange",	 remmina_main_on_action_application_mpchange,	 NULL, NULL, NULL },
 	{ "plugins",	 remmina_main_on_action_application_plugins,	 NULL, NULL, NULL },
 	{ "preferences", remmina_main_on_action_application_preferences, "i",  NULL, NULL },
+	{ "bug_report",  remmina_main_on_action_application_bug_report, NULL, NULL, NULL},
 	{ "dark",	 remmina_main_on_action_application_dark_theme,	 NULL, NULL, NULL },
 	{ "debug",	 remmina_main_on_action_help_debug,		 NULL, NULL, NULL },
 	{ "community",	 remmina_main_on_action_help_community,		 NULL, NULL, NULL },
@@ -357,10 +361,24 @@ static void remmina_main_load_file_list_callback(RemminaFile *remminafile, gpoin
 	TRACE_CALL(__func__);
 	GtkTreeIter iter;
 	GtkListStore *store;
-
+	gchar* status_icon = "";
 	store = GTK_LIST_STORE(user_data);
 	gchar *datetime;
-
+	if (remmina_pref_get_boolean("status_check")){
+		status_icon = "org.remmina.Remmina-status-grey";
+		if (g_hash_table_contains(remminamain->network_states, remminafile->filename)){
+			gchar* result = (gchar*)g_hash_table_lookup(remminamain->network_states, remminafile->filename);
+			if (result != NULL){
+				if (strncmp("Yes", result, strlen("Yes")) == 0){
+					status_icon = "org.remmina.Remmina-status-green";
+				}
+				else if (strncmp("No", result, strlen("No")) == 0){
+					status_icon = "org.remmina.Remmina-status-red";
+				}
+			}
+		}
+	}
+	
 	datetime = remmina_file_get_datetime(remminafile);
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter,
@@ -373,6 +391,7 @@ static void remmina_main_load_file_list_callback(RemminaFile *remminafile, gpoin
 			   DATE_COLUMN, datetime,
 			   FILENAME_COLUMN, remmina_file_get_filename(remminafile),
 			   LABELS_COLUMN, remmina_file_get_string(remminafile, "labels"),
+			   STATUS_COLUMN, status_icon,
 			   -1);
 	g_free(datetime);
 }
@@ -487,6 +506,21 @@ static void remmina_main_load_file_tree_callback(RemminaFile *remminafile, gpoin
 	GtkTreeStore *store;
 	gboolean found;
 	gchar *datetime = NULL;
+	gchar* status_icon = "";
+	if (remmina_pref_get_boolean("status_check")){
+		status_icon = "org.remmina.Remmina-status-grey";
+		if (g_hash_table_contains(remminamain->network_states, remminafile->filename)){
+			gchar* result = (gchar*)g_hash_table_lookup(remminamain->network_states, remminafile->filename);
+			if (result != NULL){
+				if (strncmp("Yes", result, strlen("Yes")) == 0){
+					status_icon = "org.remmina.Remmina-status-green";
+				}
+				else if (strncmp("No", result, strlen("No")) == 0){
+					status_icon = "org.remmina.Remmina-status-red";
+				}
+			}
+		}
+	}
 
 	store = GTK_TREE_STORE(user_data);
 
@@ -496,7 +530,6 @@ static void remmina_main_load_file_tree_callback(RemminaFile *remminafile, gpoin
 							 remmina_file_get_string(remminafile, "group"));
 
 	datetime = remmina_file_get_datetime(remminafile);
-	//REMMINA_DEBUG("The date is %s", datetime);
 	gtk_tree_store_append(store, &child, (found ? &iter : NULL));
 	gtk_tree_store_set(store, &child,
 			   PROTOCOL_COLUMN, remmina_file_get_icon_name(remminafile),
@@ -508,6 +541,7 @@ static void remmina_main_load_file_tree_callback(RemminaFile *remminafile, gpoin
 			   DATE_COLUMN, datetime,
 			   FILENAME_COLUMN, remmina_file_get_filename(remminafile),
 			   LABELS_COLUMN, remmina_file_get_string(remminafile, "labels"),
+			   STATUS_COLUMN, status_icon,
 			   -1);
 	g_free(datetime);
 }
@@ -676,7 +710,7 @@ static void remmina_main_load_files()
 	switch (view_file_mode) {
 	case REMMINA_VIEW_FILE_TREE:
 		/* Create new GtkTreeStore model */
-		newmodel = GTK_TREE_MODEL(gtk_tree_store_new(9, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));
+		newmodel = GTK_TREE_MODEL(gtk_tree_store_new(10, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));
 		/* Hide the Group column in the tree view mode */
 		gtk_tree_view_column_set_visible(remminamain->column_files_list_group, FALSE);
 		/* Load groups first */
@@ -688,7 +722,7 @@ static void remmina_main_load_files()
 	case REMMINA_VIEW_FILE_LIST:
 	default:
 		/* Create new GtkListStore model */
-		newmodel = GTK_TREE_MODEL(gtk_list_store_new(9, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));
+		newmodel = GTK_TREE_MODEL(gtk_list_store_new(10, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING));
 		/* Show the Group column in the list view mode */
 		gtk_tree_view_column_set_visible(remminamain->column_files_list_group, TRUE);
 		/* Load files list */
@@ -855,6 +889,7 @@ void remmina_main_on_action_connection_new(GSimpleAction *action, GVariant *para
 		return;
 	GtkWidget *widget;
 
+	remmina_plugin_manager_get_available_plugins();
 	if (remmina_pref_get_boolean("use_primary_password")
 			&& remmina_pref_get_boolean("lock_edit")
 			&& remmina_unlock_new(remminamain->window) == 0)
@@ -1079,13 +1114,15 @@ void remmina_main_reload_preferences()
 	GtkSettings *settings;
 	settings = gtk_settings_get_default();
 	g_object_set(settings, "gtk-application-prefer-dark-theme", remmina_pref.dark_theme, NULL);
-	if(remmina_pref.hide_searchbar){
-		gtk_toggle_button_set_active(remminamain->search_toggle, FALSE);
+	if (remminamain) {
+		if(remmina_pref.hide_searchbar){
+			gtk_toggle_button_set_active(remminamain->search_toggle, FALSE);
+		}
+		else{
+			gtk_toggle_button_set_active(remminamain->search_toggle, TRUE);
+		}
+		gtk_tree_view_column_set_visible(remminamain->column_files_list_notes, remmina_pref.always_show_notes);
 	}
-	else{
-		gtk_toggle_button_set_active(remminamain->search_toggle, TRUE);
-	}
-	gtk_tree_view_column_set_visible(remminamain->column_files_list_notes, remmina_pref.always_show_notes);
 }
 
 void remmina_main_on_action_application_preferences(GSimpleAction *action, GVariant *param, gpointer data)
@@ -1109,7 +1146,7 @@ void remmina_main_on_action_application_preferences(GSimpleAction *action, GVari
 
 	GtkWidget *widget = remmina_pref_dialog_new(tab_num, remminamain->window);
 
-	gtk_widget_show_all(widget);	
+	gtk_widget_show(widget);	
 }
 
 void remmina_main_on_action_application_default(GSimpleAction *action, GVariant *param, gpointer data)
@@ -1219,7 +1256,7 @@ static void remmina_main_import_file_list(GSList *files)
 		remmina_main_load_files();
 }
 
-static void remmina_main_action_tools_import_on_response(GtkDialog *dialog, gint response_id, gpointer user_data)
+static void remmina_main_action_tools_import_on_response(GtkNativeDialog *dialog, gint response_id, gpointer user_data)
 {
 	TRACE_CALL(__func__);
 	GSList *files;
@@ -1228,19 +1265,56 @@ static void remmina_main_action_tools_import_on_response(GtkDialog *dialog, gint
 		files = gtk_file_chooser_get_filenames(GTK_FILE_CHOOSER(dialog));
 		remmina_main_import_file_list(files);
 	}
-	gtk_widget_destroy(GTK_WIDGET(dialog));
+	gtk_native_dialog_destroy(dialog);
+}
+
+static void remmina_set_file_chooser_filters(GtkFileChooser *chooser)
+{
+	GtkFileFilter *filter;
+
+	g_return_if_fail(GTK_IS_FILE_CHOOSER(chooser));
+
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name(filter, _("RDP Files"));
+	gtk_file_filter_add_pattern(filter, "*.rdp");
+	gtk_file_filter_add_pattern(filter, "*.rdpx");
+	gtk_file_filter_add_pattern(filter, "*.RDP");
+	gtk_file_filter_add_pattern(filter, "*.RDPX");
+	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(chooser), filter);
+	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(chooser), filter);
+
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name(filter, _("All Files"));
+	gtk_file_filter_add_pattern(filter, "*");
+	gtk_file_chooser_add_filter(chooser, filter);
 }
 
 void remmina_main_on_action_tools_import(GSimpleAction *action, GVariant *param, gpointer data)
 {
 	TRACE_CALL(__func__);
-	GtkWidget *dialog;
+	GtkFileChooserNative *chooser;
 
-	dialog = gtk_file_chooser_dialog_new(_("Import"), remminamain->window, GTK_FILE_CHOOSER_ACTION_OPEN, "Import",
-					     GTK_RESPONSE_ACCEPT, NULL);
-	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), TRUE);
-	g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(remmina_main_action_tools_import_on_response), NULL);
-	gtk_widget_show(dialog);
+	chooser = gtk_file_chooser_native_new(_("Import"), remminamain->window,
+					      GTK_FILE_CHOOSER_ACTION_OPEN, _("Import"), _("_Cancel"));
+	gtk_native_dialog_set_modal(GTK_NATIVE_DIALOG(chooser), TRUE);
+	remmina_set_file_chooser_filters(GTK_FILE_CHOOSER(chooser));
+	gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(chooser), TRUE);
+	g_signal_connect(chooser, "response", G_CALLBACK(remmina_main_action_tools_import_on_response), NULL);
+	gtk_native_dialog_show(GTK_NATIVE_DIALOG(chooser));
+}
+
+static void on_export_save_response (GtkFileChooserNative *dialog, int response, RemminaFile *remminafile)
+{
+	if (response == GTK_RESPONSE_ACCEPT) {
+		RemminaFilePlugin *plugin = remmina_plugin_manager_get_export_file_handler(remminafile);
+		if (plugin){
+			gchar *path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));
+			plugin->export_func(plugin, remminafile, path);
+			g_free(path);
+		}
+	}
+	remmina_file_free(remminafile);
+	gtk_native_dialog_destroy(GTK_NATIVE_DIALOG(dialog));
 }
 
 void remmina_main_on_action_tools_export(GSimpleAction *action, GVariant *param, gpointer data)
@@ -1249,32 +1323,53 @@ void remmina_main_on_action_tools_export(GSimpleAction *action, GVariant *param,
 	RemminaFilePlugin *plugin;
 	RemminaFile *remminafile;
 	GtkWidget *dialog;
+	GtkFileChooserNative *chooser;
+	gchar *export_name;
 
-	if (!remminamain->priv->selected_filename)
+	if (!remminamain->priv->selected_filename) {
+		dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+						_("Select the connection profile."));
+		g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
+		gtk_widget_show(dialog);
 		return;
+	}
 
 	remminafile = remmina_file_load(remminamain->priv->selected_filename);
-	if (remminafile == NULL)
+	if (remminafile == NULL) {
+		dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+						_("Remmina couldn't export."));
+		g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
+		gtk_widget_show(dialog);
 		return;
+	}
+
 	plugin = remmina_plugin_manager_get_export_file_handler(remminafile);
 	if (plugin) {
-		dialog = gtk_file_chooser_dialog_new(plugin->export_hints, remminamain->window,
-						     GTK_FILE_CHOOSER_ACTION_SAVE, _("_Save"), GTK_RESPONSE_ACCEPT, NULL);
-		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
-			plugin->export_func(plugin, remminafile, gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog)));
-		gtk_widget_destroy(dialog);
-	} else {
+		chooser = gtk_file_chooser_native_new(plugin->export_hints, remminamain->window,
+						      GTK_FILE_CHOOSER_ACTION_SAVE, _("_Save"), _("_Cancel"));
+		gtk_native_dialog_set_modal(GTK_NATIVE_DIALOG(chooser), TRUE);
+		remmina_set_file_chooser_filters(GTK_FILE_CHOOSER(chooser));
+		gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(chooser), TRUE);
+		export_name = g_strdup_printf("%s.rdp", remminamain->priv->selected_name);
+		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(chooser), export_name);
+		g_free(export_name);
+		g_signal_connect(chooser, "response", G_CALLBACK(on_export_save_response), remminafile);
+		gtk_native_dialog_show(GTK_NATIVE_DIALOG(chooser));
+	} else
+	{
+		remmina_file_free(remminafile);
 		dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
 						_("This protocol does not support exporting."));
 		g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(gtk_widget_destroy), NULL);
 		gtk_widget_show(dialog);
+		return;
 	}
-	remmina_file_free(remminafile);
 }
 
 void remmina_main_on_action_application_plugins(GSimpleAction *action, GVariant *param, gpointer data)
 {
 	TRACE_CALL(__func__);
+	remmina_plugin_manager_get_available_plugins();
 	remmina_plugin_manager_show(remminamain->window);
 }
 
@@ -1330,6 +1425,12 @@ void remmina_main_on_action_application_about(GSimpleAction *action, GVariant *p
 	remmina_about_open(remminamain->window);
 };
 
+void remmina_main_on_action_application_bug_report(GSimpleAction *action, GVariant *param, gpointer data)
+{
+	TRACE_CALL(__func__);
+	remmina_bug_report_open(remminamain->window);
+};
+
 static gboolean is_empty(const gchar *s)
 {
 	if (s == NULL)
@@ -1369,8 +1470,6 @@ static gboolean remmina_main_quickconnect(void)
 	 */
 	if (g_strrstr(server, "/") != NULL)
 		return FALSE;
-	//if (g_str_has_suffix (server, "/"))
-	//return FALSE;
 	if (is_empty(server))
 		return FALSE;
 
@@ -1613,8 +1712,6 @@ static void remmina_main_init(void)
 	}
 	gtk_combo_box_set_active(GTK_COMBO_BOX(remminamain->combo_quick_connect_protocol), qcp_actidx);
 
-	/* Connect the group accelerators to the GtkWindow */
-	//gtk_window_add_accel_group(remminamain->window, remminamain->accelgroup_shortcuts);
 	/* Set the Quick Connection */
 	gtk_entry_set_activates_default(remminamain->entry_quick_connect_server, TRUE);
 	/* Set the TreeView for the files list */
@@ -1648,6 +1745,15 @@ void remmina_main_on_show(GtkWidget *w, gpointer user_data)
 #endif
 }
 
+void remmina_main_add_network_status(gchar* key, gchar* value)
+{
+	if (remminamain != NULL){
+		g_hash_table_insert(remminamain->network_states, key, value);
+		remmina_main_load_files();
+	}
+	
+}
+
 /* RemminaMain instance */
 GtkWidget *remmina_main_new(void)
 {
@@ -1657,6 +1763,7 @@ GtkWidget *remmina_main_new(void)
 
 	remminamain = g_new0(RemminaMain, 1);
 	remminamain->priv = g_new0(RemminaMainPriv, 1);
+	remminamain->network_states = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 	/* Assign UI widgets to the private members */
 	remminamain->builder = remmina_public_gtk_builder_new_from_resource("/org/remmina/Remmina/src/../data/ui/remmina_main.glade");
 	remminamain->window = GTK_WINDOW(RM_GET_OBJECT("RemminaMain"));
