@@ -105,7 +105,8 @@ struct _RemminaProtocolWidgetPriv {
 enum panel_type {
 	RPWDT_AUTH,
 	RPWDT_QUESTIONYESNO,
-	RPWDT_AUTHX509
+	RPWDT_AUTHX509,
+	RPWDT_ACCEPT
 };
 
 G_DEFINE_TYPE(RemminaProtocolWidget, remmina_protocol_widget, GTK_TYPE_EVENT_BOX)
@@ -631,6 +632,9 @@ void remmina_protocol_widget_send_clip_strokes(GtkClipboard *clipboard, const gc
 {
 	TRACE_CALL(__func__);
 	RemminaProtocolWidget *gp = REMMINA_PROTOCOL_WIDGET(data);
+	if (clip_text == NULL){
+		return;
+	}
 	gchar *text = g_utf8_normalize(clip_text, -1, G_NORMALIZE_DEFAULT_COMPOSE);
 	guint *keyvals;
 	gint i;
@@ -1620,7 +1624,9 @@ static gboolean remmina_protocol_widget_dialog_mt_setup(gpointer user_data)
 		if (d->pflags & REMMINA_MESSAGE_PANEL_FLAG_SAVEPASSWORD)
 			remmina_message_panel_field_set_switch(mp, REMMINA_MESSAGE_PANEL_FLAG_SAVEPASSWORD, (d->default_password == NULL || d->default_password[0] == 0) ? FALSE: TRUE);
 	} else if (d->dtype == RPWDT_QUESTIONYESNO) {
-		remmina_message_panel_setup_question(mp, d->title, authpanel_mt_cb, d);
+		remmina_message_panel_setup_question(mp, d->title, authpanel_mt_cb, d, FALSE);
+	} else if (d->dtype == RPWDT_ACCEPT) {
+		remmina_message_panel_setup_question(mp, d->title, authpanel_mt_cb, d, TRUE);
 	} else if (d->dtype == RPWDT_AUTHX509) {
 		remmina_message_panel_setup_auth_x509(mp, authpanel_mt_cb, d);
 		if ((s = remmina_file_get_string(remminafile, "cacert")) != NULL)
@@ -1732,13 +1738,11 @@ static int remmina_protocol_widget_dialog(enum panel_type dtype, RemminaProtocol
 		rcbutton = mpri.response;
 	} else {
 		d->called_from_subthread = TRUE;
-		// pthread_cleanup_push(ptcleanup, (void*)d);
 		pthread_cond_init(&d->pt_cond, NULL);
 		pthread_mutex_init(&d->pt_mutex, NULL);
 		g_idle_add(remmina_protocol_widget_dialog_mt_setup, d);
 		pthread_mutex_lock(&d->pt_mutex);
 		pthread_cond_wait(&d->pt_cond, &d->pt_mutex);
-		// pthread_cleanup_pop(0);
 		pthread_mutex_destroy(&d->pt_mutex);
 		pthread_cond_destroy(&d->pt_cond);
 
@@ -1759,6 +1763,11 @@ gint remmina_protocol_widget_panel_question_yesno(RemminaProtocolWidget *gp, con
 	return remmina_protocol_widget_dialog(RPWDT_QUESTIONYESNO, gp, 0, msg, NULL, NULL, NULL, NULL);
 }
 
+gint remmina_protocol_widget_panel_question_accept(RemminaProtocolWidget *gp, const char *msg)
+{
+	return remmina_protocol_widget_dialog(RPWDT_ACCEPT, gp, 0, msg, NULL, NULL, NULL, NULL);
+}
+
 gint remmina_protocol_widget_panel_auth(RemminaProtocolWidget *gp, RemminaMessagePanelFlags pflags,
 					const gchar *title, const gchar *default_username, const gchar *default_password, const gchar *default_domain, const gchar *password_prompt)
 {
